@@ -6,6 +6,10 @@ IconWidget::IconWidget(QWidget *parent)
       pixmap(nullptr)
 {
     dpr = this->devicePixelRatioF();
+    connect(settings, &Settings::settingsChanged, [this]() {
+        if(pixmap)
+            ImageLib::recolor(*(this->pixmap), settings->colorScheme().icons);
+    });
 }
 
 IconWidget::~IconWidget() {
@@ -30,14 +34,29 @@ void IconWidget::setIconPath(QString path) {
         pixmap = new QPixmap(path);
         pixmapDrawScale = dpr;
     }
+    // check if the pixmap is valid
+    if(!pixmap->isNull()) {
+        ImageLib::recolor(*pixmap, settings->colorScheme().icons);
+    } else {
+        delete pixmap;
+        pixmap = nullptr;
+    }
+    update();
+}
+
+void IconWidget::setIconOffset(int x, int y) {
+    iconOffset.setX(x);
+    iconOffset.setY(y);
     update();
 }
 
 void IconWidget::paintEvent(QPaintEvent *event) {
     Q_UNUSED(event)
     QPainter p(this);
+    if(!this->isEnabled())
+        p.setOpacity(0.5f);
     QStyleOption opt;
-    opt.init(this);
+    opt.initFrom(this);
     style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
     if(pixmap) {
         p.setRenderHint(QPainter::SmoothPixmapTransform);
@@ -49,6 +68,6 @@ void IconWidget::paintEvent(QPaintEvent *event) {
             pos = QPointF(width()  / 2 - pixmap->width()  / 2,
                           height() / 2 - pixmap->height() / 2);
         }
-        p.drawPixmap(pos, *pixmap);
+        p.drawPixmap(pos + iconOffset, *pixmap);
     }
 }
