@@ -5,14 +5,16 @@ PathSelectorMenuItem::PathSelectorMenuItem(QWidget *parent)
       mDirectory("")
 {
     setFocusPolicy(Qt::NoFocus);
-    setIcon(QIcon(":/res/icons/menuitem/folder16.png"));
-    mIconLabel.setAttribute(Qt::WA_TransparentForMouseEvents, false);
-    connect(&mIconLabel, &ClickableLabel::clicked, [this]() {
+    setIconPath(":/res/icons/common/menuitem/folder16.png");
+    mIconWidget.setAttribute(Qt::WA_TransparentForMouseEvents, false);
+    connect(&mIconWidget, &IconButton::clicked, [this]() {
         QFileDialog dialog;
         dialog.setDirectory(mDirectory);
-        dialog.setFileMode(QFileDialog::Directory);
         dialog.setWindowTitle("Select directory");
         dialog.setWindowModality(Qt::ApplicationModal);
+        dialog.setFileMode(QFileDialog::Directory);
+        dialog.setOption(QFileDialog::ShowDirsOnly);
+        dialog.setOption(QFileDialog::DontResolveSymlinks);
         connect(&dialog, &QFileDialog::fileSelected, this, &PathSelectorMenuItem::setDirectory);
         dialog.exec();
     });
@@ -24,7 +26,12 @@ QString PathSelectorMenuItem::directory() {
 
 void PathSelectorMenuItem::setDirectory(QString path) {
     mDirectory = path;
-    QString stripped = mDirectory.split("/", QString::SkipEmptyParts).last();
+    QString stripped;
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    stripped = mDirectory.split("/").last();
+#else
+    stripped = mDirectory.split("/", Qt::SkipEmptyParts).last();
+#endif
     this->mTextLabel.setText(stripped);
 }
 
