@@ -2,6 +2,7 @@
 
 #include <QFileDialog>
 #include "gui/customwidgets/menuitem.h"
+#include "settings.h"
 
 class PathSelectorMenuItem : public MenuItem {
     Q_OBJECT
@@ -13,7 +14,7 @@ public slots:
     void setDirectory(QString path);
 
 protected:
-    virtual void onPress() Q_DECL_OVERRIDE;
+    virtual void onPress() override;
 
 signals:
     void directorySelected(QString);
