@@ -5,8 +5,6 @@
 MW::MW(QWidget *parent)
     : FloatingWidgetContainer(parent),
       currentDisplay(0),
-      desktopWidget(nullptr),
-      bgOpacity(1.0),
       maximized(false),
       activeSidePanel(SIDEPANEL_NONE),
       copyOverlay(nullptr),
@@ -36,7 +34,6 @@ MW::MW(QWidget *parent)
     this->setMouseTracking(true);
     this->setAcceptDrops(true);
     this->setAccessibleName("mainwindow");
-    desktopWidget = QApplication::desktop();
     windowGeometryChangeTimer.setSingleShot(true);
     windowGeometryChangeTimer.setInterval(30);
     setupUi();
@@ -66,9 +63,9 @@ void MW::setupUi() {
     folderView.reset(new FolderViewProxy(this));
     connect(folderView.get(), &FolderViewProxy::sortingSelected, this, &MW::sortingSelected);
     connect(folderView.get(), &FolderViewProxy::directorySelected, this, &MW::opened);
-    connect(folderView.get(), &FolderViewProxy::draggedOut, this, qOverload<int>(&MW::draggedOut));
     connect(folderView.get(), &FolderViewProxy::copyUrlsRequested, this, &MW::copyUrlsRequested);
     connect(folderView.get(), &FolderViewProxy::moveUrlsRequested, this, &MW::moveUrlsRequested);
+    connect(folderView.get(), &FolderViewProxy::showFoldersChanged, this, &MW::showFoldersChanged);
 
     centralWidget.reset(new CentralWidget(docWidget, folderView, this));
     layout.addWidget(centralWidget.get());
@@ -91,20 +88,20 @@ void MW::setupUi() {
     connect(this, &MW::scrollRight, viewerWidget.get(), &ViewerWidget::scrollRight);
     connect(this, &MW::pauseVideo,     viewerWidget.get(), &ViewerWidget::pauseResumePlayback);
     connect(this, &MW::stopPlayback,   viewerWidget.get(), &ViewerWidget::stopPlayback);
-    connect(this, &MW::seekVideoRight, viewerWidget.get(), &ViewerWidget::seekRight);
-    connect(this, &MW::seekVideoLeft,  viewerWidget.get(), &ViewerWidget::seekLeft);
+    connect(this, &MW::seekVideoForward, viewerWidget.get(), &ViewerWidget::seekForward);
+    connect(this, &MW::seekVideoBackward,  viewerWidget.get(), &ViewerWidget::seekBackward);
     connect(this, &MW::frameStep,      viewerWidget.get(), &ViewerWidget::frameStep);
     connect(this, &MW::frameStepBack,  viewerWidget.get(), &ViewerWidget::frameStepBack);
     connect(this, &MW::toggleMute,  viewerWidget.get(), &ViewerWidget::toggleMute);
     connect(this, &MW::volumeUp,  viewerWidget.get(), &ViewerWidget::volumeUp);
     connect(this, &MW::volumeDown,  viewerWidget.get(), &ViewerWidget::volumeDown);
     connect(this, &MW::toggleTransparencyGrid, viewerWidget.get(), &ViewerWidget::toggleTransparencyGrid);
-    connect(this, &MW::enableDocumentView, centralWidget.get(), &CentralWidget::showDocumentView);
     connect(this, &MW::setLoopPlayback,  viewerWidget.get(), &ViewerWidget::setLoopPlayback);
 }
 
 void MW::setupFullUi() {
     setupCropPanel();
+    viewerWidget->setupMainPanel();
     infoBarWindowed->init();
     infoBarFullscreen->init();
 }
@@ -135,7 +132,7 @@ void MW::setupSaveOverlay() {
 }
 
 void MW::setupRenameOverlay() {
-    renameOverlay = new RenameOverlay(viewerWidget.get());
+    renameOverlay = new RenameOverlay(this);
     renameOverlay->setName(info.fileName);
     connect(renameOverlay, &RenameOverlay::renameRequested, this, &MW::renameRequested);
 }
@@ -149,6 +146,7 @@ void MW::toggleFolderView() {
     viewerWidget->hidePanel();
     imageInfoOverlay->hide();
     centralWidget->toggleViewMode();
+    onInfoUpdated();
 }
 
 void MW::enableFolderView() {
@@ -163,12 +161,13 @@ void MW::enableFolderView() {
     onInfoUpdated();
 }
 
-ViewMode MW::currentViewMode() {
-    return centralWidget->currentViewMode();
+void MW::enableDocumentView() {
+    centralWidget->showDocumentView();
+    onInfoUpdated();
 }
 
-int MW::folderViewSelection() {
-    return folderView->selectedIndex();
+ViewMode MW::currentViewMode() {
+    return centralWidget->currentViewMode();
 }
 
 void MW::fitWindow() {
@@ -206,6 +205,7 @@ void MW::switchFitMode() {
 
 void MW::closeImage() {
     info.fileName = "";
+    info.filePath = "";
     viewerWidget->closeImage();
 }
 
@@ -214,8 +214,8 @@ void MW::setImage(std::unique_ptr<QPixmap> pixmap) {
     updateCropPanelData();
 }
 
-void MW::setAnimation(std::unique_ptr<QMovie> movie) {
-    viewerWidget->showAnimation(std::move(movie));
+void MW::setAnimation(std::shared_ptr<QMovie> movie) {
+    viewerWidget->showAnimation(movie);
     updateCropPanelData();
 }
 
@@ -243,11 +243,40 @@ void MW::onSortingChanged(SortingMode mode) {
 
 void MW::setDirectoryPath(QString path) {
     closeImage();
-    info.directory = path;
+    info.directoryPath = path;
+    info.directoryName = path.split("/").last();
     folderView->setDirectoryPath(path);
     onInfoUpdated();
 }
 
+void MW::toggleLockZoom() {
+    viewerWidget->toggleLockZoom();
+    if(viewerWidget->lockZoomEnabled())
+        showMessage("Zoom lock: ON");
+    else
+        showMessage("Zoom lock: OFF");
+    onInfoUpdated();
+}
+
+void MW::toggleLockView() {
+    viewerWidget->toggleLockView();
+    if(viewerWidget->lockViewEnabled())
+        showMessage("View lock: ON");
+    else
+        showMessage("View lock: OFF");
+    onInfoUpdated();
+}
+
+void MW::toggleFullscreenInfoBar() {
+    if(!this->isFullScreen())
+        return;
+    showInfoBarFullscreen = !showInfoBarFullscreen;
+    if(showInfoBarFullscreen)
+        infoBarFullscreen->showWhenReady();
+    else
+        infoBarFullscreen->hide();
+}
+
 void MW::toggleImageInfoOverlay() {
     if(centralWidget->currentViewMode() == MODE_FOLDERVIEW)
         return;
@@ -257,15 +286,16 @@ void MW::toggleImageInfoOverlay() {
         imageInfoOverlay->hide();
 }
 
-void MW::toggleRenameOverlay() {
+void MW::toggleRenameOverlay(QString currentName) {
     if(!renameOverlay)
         setupRenameOverlay();
-    if(centralWidget->currentViewMode() == MODE_FOLDERVIEW)
-        return;
-    if(renameOverlay->isHidden())
+    if(renameOverlay->isHidden()) {
+        renameOverlay->setBackdropEnabled((centralWidget->currentViewMode() == MODE_FOLDERVIEW));
+        renameOverlay->setName(currentName);
         renameOverlay->show();
-    else
+    } else {
         renameOverlay->hide();
+    }
 }
 
 void MW::toggleScalingFilter() {
@@ -294,29 +324,26 @@ void MW::onScalingFinished(std::unique_ptr<QPixmap> scaled) {
 }
 
 void MW::saveWindowGeometry() {
-    if(this->windowState() == Qt::WindowNoState) {
-    #ifdef __linux__
-        windowedGeometry = geometry();
-    #else
-        windowedGeometry = QRect(pos(), size());
-    #endif
-    }
-    settings->setWindowGeometry(windowedGeometry);
+    if(this->windowState() == Qt::WindowNoState)
+        settings->setWindowGeometry(geometry());
     settings->setMaximizedWindow(maximized);
 }
 
 // does not apply fullscreen; window size / maximized state only
 void MW::restoreWindowGeometry() {
-    windowedGeometry = settings->windowGeometry();
-    this->resize(windowedGeometry.size());
-    this->move(windowedGeometry.x(), windowedGeometry.y());
+    this->setGeometry(settings->windowGeometry());
     if(settings->maximizedWindow())
         this->setWindowState(Qt::WindowMaximized);
     updateCurrentDisplay();
 }
 
 void MW::updateCurrentDisplay() {
-    currentDisplay = desktopWidget->screenNumber(this);
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    currentDisplay = desktopWidget.screenNumber(this);
+#else
+    auto screens = qApp->screens();
+    currentDisplay = screens.indexOf(this->window()->screen());
+#endif
 }
 
 void MW::onWindowGeometryChanged() {
@@ -325,7 +352,11 @@ void MW::onWindowGeometryChanged() {
 }
 
 void MW::saveCurrentDisplay() {
-    settings->setLastDisplay(desktopWidget->screenNumber(this));
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    settings->setLastDisplay(desktopWidget.screenNumber(this));
+#else
+    settings->setLastDisplay(qApp->screens().indexOf(this->window()->screen()));
+#endif
 }
 
 //#############################################################
@@ -418,13 +449,46 @@ void MW::showDefault() {
 
 void MW::showSaveDialog(QString filePath) {
     viewerWidget->hidePanel();
+    QStringList filters;
+    // generate filter for writable images
+    // todo: some may need to be blacklisted
+    auto writerFormats = QImageWriter::supportedImageFormats();
+    if(writerFormats.contains("jpg"))  filters.append("JPEG (*.jpg *.jpeg *jpe *jfif)");
+    if(writerFormats.contains("png"))  filters.append("PNG (*.png)");
+    if(writerFormats.contains("webp")) filters.append("WebP (*.webp)");
+    // may not work..
+    if(writerFormats.contains("jp2"))  filters.append("JPEG 2000 (*.jp2 *.j2k *.jpf *.jpx *.jpm *.jpgx)");
+    if(writerFormats.contains("jxl"))  filters.append("JPEG-XL (*.jxl)");
+    if(writerFormats.contains("avif")) filters.append("AVIF (*.avif *.avifs)");
+    if(writerFormats.contains("tif"))  filters.append("TIFF (*.tif *.tiff)");
+    if(writerFormats.contains("bmp"))  filters.append("BMP (*.bmp)");
+#ifdef _WIN32
+    if(writerFormats.contains("ico"))  filters.append("Icon Files (*.ico)");
+#endif
+    if(writerFormats.contains("ppm"))  filters.append("PPM (*.ppm)");
+    if(writerFormats.contains("xbm"))  filters.append("XBM (*.xbm)");
+    if(writerFormats.contains("xpm"))  filters.append("XPM (*.xpm)");
+    if(writerFormats.contains("dds"))  filters.append("DDS (*.dds)");
+    if(writerFormats.contains("wbmp")) filters.append("WBMP (*.wbmp)");
+    // add everything else from imagewriter
+    for(auto fmt : writerFormats) {
+        if(filters.filter(fmt).isEmpty())
+            filters.append(fmt.toUpper() + " (*." + fmt + ")");
+    }
+    QString filterString = filters.join(";; ");
 
-    const QString imagesFilter = settings->supportedFormatsString();
-    filePath = QFileDialog::getSaveFileName(this, tr("Save File"),
-                                            filePath,
-                                            imagesFilter);
-    if(!filePath.isEmpty())
-        emit saveAsRequested(filePath);
+    // find matching filter for the current image
+    QString selectedFilter = "JPEG (*.jpg *.jpeg *jpe *jfif)";
+    QFileInfo fi(filePath);
+    for(auto filter : filters) {
+        if(filter.contains(fi.suffix().toLower())) {
+            selectedFilter = filter;
+            break;
+        }
+    }
+    QString newFilePath = QFileDialog::getSaveFileName(this, tr("Save File as..."), filePath, filterString, &selectedFilter);
+    if(!newFilePath.isEmpty())
+        emit saveAsRequested(newFilePath);
 }
 
 void MW::showOpenDialog(QString path) {
@@ -432,7 +496,7 @@ void MW::showOpenDialog(QString path) {
 
     QFileDialog dialog(this);
     QStringList imageFilter;
-    imageFilter.append(settings->supportedFormatsString());
+    imageFilter.append(settings->supportedFormatsFilter());
     imageFilter.append("All Files (*)");
     dialog.setDirectory(path);
     dialog.setNameFilters(imageFilter);
@@ -448,6 +512,19 @@ void MW::showResizeDialog(QSize initialSize) {
     dialog.exec();
 }
 
+DialogResult MW::fileReplaceDialog(QString src, QString dst, FileReplaceMode mode, bool multiple) {
+    FileReplaceDialog dialog(this);
+    dialog.setModal(true);
+    dialog.setSource(src);
+    dialog.setDestination(dst);
+    dialog.setMode(mode);
+    dialog.setMulti(multiple);
+
+    dialog.exec();
+
+    return dialog.getResult();
+}
+
 void MW::showSettings() {
     viewerWidget->hidePanel();
 
@@ -467,14 +544,21 @@ void MW::showFullScreen() {
     //do not save immediately on application start
     if(!isHidden())
         saveWindowGeometry();
+    auto screens = qApp->screens();
+    // todo: why check the screen again?
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    int _currentDisplay = desktopWidget.screenNumber(this);
+#else
+    int _currentDisplay = screens.indexOf(this->window()->screen());
+#endif
     //move to target screen
-    if(desktopWidget->screenCount() > currentDisplay &&
-       currentDisplay != desktopWidget->screenNumber(this))
-    {
-        this->move(desktopWidget->screenGeometry(currentDisplay).x(),
-                   desktopWidget->screenGeometry(currentDisplay).y());
+    if(screens.count() > currentDisplay && currentDisplay != _currentDisplay) {
+        this->move(screens.at(currentDisplay)->geometry().x(),
+                   screens.at(currentDisplay)->geometry().y());
     }
     QWidget::showFullScreen();
+    // try to repaint sooner
+    qApp->processEvents();
     emit fullscreenStateChanged(true);
 }
 
@@ -483,6 +567,8 @@ void MW::showWindowed() {
         QWidget::showNormal();
     restoreWindowGeometry();
     QWidget::show();
+    // try to repaint sooner
+    qApp->processEvents();
     emit fullscreenStateChanged(false);
 }
 
@@ -496,6 +582,8 @@ void MW::updateCropPanelData() {
 }
 
 void MW::showSaveOverlay() {
+    if(!settings->showSaveOverlay())
+        return;
     if(!saveOverlay)
         setupSaveOverlay();
     saveOverlay->show();
@@ -592,13 +680,17 @@ void MW::closeFullScreenOrExit() {
     }
 }
 
-void MW::setCurrentInfo(int _index, int _fileCount, QString _fileName, QSize _imageSize, qint64 _fileSize, bool slideshow) {
+// todo: this is crap, use shared state object
+void MW::setCurrentInfo(int _index, int _fileCount, QString _filePath, QString _fileName, QSize _imageSize, qint64 _fileSize, bool slideshow, bool shuffle, bool edited) {
     info.index = _index;
     info.fileCount = _fileCount;
     info.fileName = _fileName;
+    info.filePath = _filePath;
     info.imageSize = _imageSize;
     info.fileSize = _fileSize;
     info.slideshow = slideshow;
+    info.shuffle = shuffle;
+    info.edited = edited;
     onInfoUpdated();
 }
 
@@ -611,20 +703,15 @@ void MW::onInfoUpdated() {
     if(info.imageSize.width())
         resString = QString::number(info.imageSize.width()) + " x " + QString::number(info.imageSize.height());
     QString sizeString;
-    if(info.fileSize) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-        sizeString = QString::number(info.fileSize / 1024) + " KiB";
-#else
+    if(info.fileSize)
         sizeString = this->locale().formattedDataSize(info.fileSize, 1);
-#endif
-    }
 
     if(renameOverlay)
         renameOverlay->setName(info.fileName);
 
     QString windowTitle;
     if(centralWidget->currentViewMode() == MODE_FOLDERVIEW) {
-        windowTitle = info.directory + " — " + qApp->applicationName();
+        windowTitle = "Folder view";
         infoBarFullscreen->setInfo("", "No file opened.", "");
         infoBarWindowed->setInfo("", "No file opened.", "");
     } else if(info.fileName.isEmpty()) {
@@ -640,10 +727,25 @@ void MW::onInfoUpdated() {
             if(!sizeString.isEmpty())
                 windowTitle.append("  -  " + sizeString);
         }
+
+        // toggleable states
+        QString states;
         if(info.slideshow)
-            windowTitle.append(" — slideshow");
-        infoBarFullscreen->setInfo(posString, info.fileName, resString + "  " + sizeString);
-        infoBarWindowed->setInfo(posString, info.fileName, resString + "  " + sizeString);
+            states.append(" [slideshow]");
+        if(info.shuffle)
+            states.append(" [shuffle]");
+        if(viewerWidget->lockZoomEnabled())
+            states.append(" [zoom lock]");
+        if(viewerWidget->lockViewEnabled())
+            states.append(" [view lock]");
+
+        if(!settings->infoBarWindowed() && !states.isEmpty())
+            windowTitle.append(" -" + states);
+        if(info.edited)
+            windowTitle.prepend("* ");
+
+        infoBarFullscreen->setInfo(posString, info.fileName + (info.edited ? "  *" : ""), resString + "  " + sizeString);
+        infoBarWindowed->setInfo(posString, info.fileName + (info.edited ? "  *" : ""), resString + "  " + sizeString + " " + states);
     }
     setWindowTitle(windowTitle);
 }
@@ -658,10 +760,15 @@ std::shared_ptr<FolderViewProxy> MW::getFolderView() {
     return folderView;
 }
 
-std::shared_ptr<ThumbnailStrip> MW::getThumbnailPanel() {
+std::shared_ptr<ThumbnailStripProxy> MW::getThumbnailPanel() {
     return viewerWidget->getThumbPanel();
 }
 
+// todo: this is crap
+void MW::showMessageDirectory(QString dirName) {
+    floatingMessage->showMessage(dirName, FloatingMessageIcon::ICON_DIRECTORY, 1700);
+}
+
 void MW::showMessageDirectoryEnd() {
     floatingMessage->showMessage("", FloatingWidgetPosition::RIGHT, FloatingMessageIcon::ICON_RIGHT_EDGE, 400);
 }
@@ -699,7 +806,22 @@ void MW::showWarning(QString text) {
 }
 
 void MW::showError(QString text) {
-    floatingMessage->showMessage(text,  FloatingMessageIcon::ICON_ERROR, 2500);
+    floatingMessage->showMessage(text,  FloatingMessageIcon::ICON_ERROR, 2800);
+}
+
+bool MW::showConfirmation(QString title, QString msg) {
+    QMessageBox msgBox(this);
+    msgBox.setWindowTitle(title);
+    msgBox.setText(msg);
+    msgBox.setIcon(QMessageBox::Warning);
+    msgBox.setStandardButtons(QMessageBox::Yes);
+    msgBox.addButton(QMessageBox::No);
+    msgBox.setDefaultButton(QMessageBox::Yes);
+    msgBox.setModal(true);
+    if(msgBox.exec() == QMessageBox::Yes)
+        return true;
+    else
+        return false;
 }
 
 void MW::readSettings() {
@@ -708,11 +830,10 @@ void MW::readSettings() {
     adaptToWindowState();
 }
 
+// todo: remove/rename?
 void MW::applyWindowedBackground() {
-    bgColor = settings->backgroundColor();
-    bgOpacity = settings->backgroundOpacity();
 #ifdef USE_KDE_BLUR
-    if(bgOpacity == 1.0)
+    if(settings->backgroundOpacity() == 1.0)
         KWindowEffects::enableBlurBehind(winId(), false);
     else
         KWindowEffects::enableBlurBehind(winId(), settings->blurBackground());
@@ -720,8 +841,6 @@ void MW::applyWindowedBackground() {
 }
 
 void MW::applyFullscreenBackground() {
-    bgColor = settings->backgroundColorFullscreen();
-    bgOpacity = 1.0;
 #ifdef USE_KDE_BLUR
     KWindowEffects::enableBlurBehind(winId(), false);
 #endif
@@ -734,9 +853,9 @@ void MW::adaptToWindowState() {
         applyFullscreenBackground();
         infoBarWindowed->hide();
         if(showInfoBarFullscreen)
-            infoBarFullscreen->show();
+            infoBarFullscreen->showWhenReady();
         else
-            infoBarFullscreen->hide();
+            infoBarFullscreen->hide();    
         if(viewerWidget->panelPosition() == PANEL_BOTTOM || !viewerWidget->panelEnabled())
             controlsOverlay->show();
         else
@@ -755,12 +874,8 @@ void MW::adaptToWindowState() {
 }
 
 void MW::paintEvent(QPaintEvent *event) {
-    /*
     QPainter p(this);
-    p.setOpacity(bgOpacity);
-    p.setBrush(QBrush(bgColor));
-    p.fillRect(this->rect(), p.brush());
-    */
+    p.fillRect(rect(), Qt::black);
     FloatingWidgetContainer::paintEvent(event);
 }
 
@@ -768,3 +883,8 @@ void MW::leaveEvent(QEvent *event) {
     QWidget::leaveEvent(event);
     viewerWidget->hidePanelAnimated();
 }
+
+// block native tab-switching so we can use it in shortcuts
+//bool MW::focusNextPrevChild(bool) {
+//    return false;
+//}
