@@ -12,6 +12,7 @@
 #include <cmath>
 
 #include "gui/customwidgets/thumbnailview.h"
+#include "gui/customwidgets/thumbnailwidget.h"
 #include "sourcecontainers/thumbnail.h"
 
 class ThumbnailStrip : public ThumbnailView
@@ -19,21 +20,18 @@ class ThumbnailStrip : public ThumbnailView
     Q_OBJECT
 public:
     explicit ThumbnailStrip(QWidget *parent = nullptr);
+    QSize itemSize();
+    void readSettings();
 
 private:
-    int panelSize;
-
-    int thumbnailSpacing;
-
+    int thumbPadding = 9, thumbMarginX = 2, thumbMarginY = 4;
     void updateThumbnailPositions(int start, int end);
     void updateThumbnailPositions();
-    void setThumbnailSize(int);
-    void updateThumbnailSize();
     void setupLayout();
+    ThumbnailStyle mCurrentStyle;
 
 public slots:
     virtual void focusOn(int index);
-    void readSettings();
 
 protected:
     virtual void resizeEvent(QResizeEvent *event);
