@@ -12,17 +12,17 @@ ImageViewerV2::ImageViewerV2(QWidget *parent) : QGraphicsView(parent),
     keepFitMode(false),
     loopPlayback(true),
     mIsFullscreen(false),
+    absoluteStep(false),
     mouseInteraction(MouseInteractionState::MOUSE_NONE),
     minScale(0.01f),
     maxScale(500.0f),
+    fitWindowScale(0.125f),
+    mViewLock(LOCK_NONE),
     imageFitMode(FIT_WINDOW),
-    imageFitModeDefault(FIT_WINDOW),
     mScalingFilter(QI_FILTER_BILINEAR),
+    imageFitModeDefault(FIT_WINDOW),
     scene(nullptr)
 {
-    if(settings->useOpenGL())
-        setViewport(new QOpenGLWidget);
-
     setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate);
     this->viewport()->setAttribute(Qt::WA_OpaquePaintEvent, true);
     setFocusPolicy(Qt::NoFocus);
@@ -38,6 +38,8 @@ ImageViewerV2::ImageViewerV2(QWidget *parent) : QGraphicsView(parent),
     scrollTimeLineX->setEasingCurve(QEasingCurve::OutSine);
     scrollTimeLineX->setDuration(ANIMATION_SPEED);
     scrollTimeLineX->setUpdateInterval(SCROLL_UPDATE_RATE);
+    connect(scrollTimeLineX, &QTimeLine::finished, this, &ImageViewerV2::onScrollTimelineFinished);
+    connect(scrollTimeLineY, &QTimeLine::finished, this, &ImageViewerV2::onScrollTimelineFinished);
 
     animationTimer = new QTimer(this);
     animationTimer->setSingleShot(true);
@@ -95,8 +97,10 @@ void ImageViewerV2::readSettings() {
     zoomStep = settings->zoomStep();
     transparencyGridEnabled = settings->transparencyGrid();
     focusIn1to1 = settings->focusPointIn1to1Mode();
+    absoluteStep = settings->absoluteZoomStep();
     // set bg color
     onFullscreenModeChanged(mIsFullscreen);
+    updateMinScale();
     setScalingFilter(settings->scalingFilter());
     setFitMode(imageFitModeDefault);
 }
@@ -105,10 +109,10 @@ void ImageViewerV2::onFullscreenModeChanged(bool mode) {
     QColor bgColor;
     mIsFullscreen = mode;
     if(mode) {
-        bgColor = settings->backgroundColorFullscreen();
+        bgColor = settings->colorScheme().background_fullscreen;
         bgColor.setAlphaF(1.0);
     } else {
-        bgColor = settings->backgroundColor();
+        bgColor = settings->colorScheme().background;
         bgColor.setAlphaF(settings->backgroundOpacity());
     }
     scene->setBackgroundBrush(bgColor);
@@ -153,7 +157,11 @@ void ImageViewerV2::onAnimationTimer() {
             movie->jumpToFrame(0);
         }
     } else {
-        movie->jumpToNextFrame();
+        if(!movie->jumpToNextFrame()) {
+            qDebug() << "[Error] QMovie:" << movie->lastErrorString();
+            this->stopAnimation();
+            return;
+        }
     }
     emit frameChanged(movie->currentFrameNumber());
     std::unique_ptr<QPixmap> newFrame(new QPixmap());
@@ -163,15 +171,23 @@ void ImageViewerV2::onAnimationTimer() {
 }
 
 void ImageViewerV2::nextFrame() {
-    if(!movie || movie->currentFrameNumber() == movie->frameCount() - 1)
+    if(!movie) {
         return;
-    showAnimationFrame(movie->currentFrameNumber() + 1);
+    } else if(movie->currentFrameNumber() == movie->frameCount() - 1) {
+        showAnimationFrame(0);
+    } else {
+        showAnimationFrame(movie->currentFrameNumber() + 1);
+    }
 }
 
 void ImageViewerV2::prevFrame() {
-    if(!movie || movie->currentFrameNumber() == 0)
+    if(!movie) {
         return;
-    showAnimationFrame(movie->currentFrameNumber() - 1);
+    } else if(movie->currentFrameNumber() == 0) {
+        showAnimationFrame(movie->frameCount() - 1);
+    } else {
+        showAnimationFrame(movie->currentFrameNumber() - 1);
+    }
 }
 
 bool ImageViewerV2::showAnimationFrame(int frame) {
@@ -186,7 +202,10 @@ bool ImageViewerV2::showAnimationFrame(int frame) {
     if(frame < movie->currentFrameNumber())
         movie->jumpToFrame(0);
     while(frame != movie->currentFrameNumber()) {
-        movie->jumpToNextFrame();
+        if(!movie->jumpToNextFrame()) {
+            qDebug() << "[Error] QMovie:" << movie->lastErrorString();
+            break;
+        }
     }
     emit frameChanged(movie->currentFrameNumber());
     std::unique_ptr<QPixmap> newFrame(new QPixmap());
@@ -208,10 +227,10 @@ void ImageViewerV2::updatePixmap(std::unique_ptr<QPixmap> newPixmap) {
     pixmapItem.update();
 }
 
-void ImageViewerV2::displayAnimation(std::unique_ptr<QMovie> _movie) {
+void ImageViewerV2::displayAnimation(std::shared_ptr<QMovie> _movie) {
     if(_movie && _movie->isValid()) {
         reset();
-        movie = std::move(_movie);
+        movie = _movie;
         movie->jumpToFrame(0);
         Qt::TransformationMode mode = smoothAnimatedImages ? Qt::SmoothTransformation : Qt::FastTransformation;
         pixmapItem.setTransformationMode(mode);
@@ -221,11 +240,18 @@ void ImageViewerV2::displayAnimation(std::unique_ptr<QMovie> _movie) {
         emit durationChanged(movie->frameCount());
         emit frameChanged(0);
 
-        if(!keepFitMode)
-                imageFitMode = imageFitModeDefault;
-        if(imageFitMode == FIT_FREE)
-            imageFitMode = FIT_WINDOW;
-        applyFitMode();
+        updateMinScale();
+        if(!keepFitMode || imageFitMode == FIT_FREE)
+            imageFitMode = imageFitModeDefault;
+
+        if(mViewLock == LOCK_NONE) {
+            applyFitMode();
+        } else {
+            imageFitMode = FIT_FREE;
+            fitFree(lockedScale);
+            if(mViewLock == LOCK_ALL)
+                applySavedViewportPos();
+        }
 
         if(transparencyGridEnabled)
             drawTransparencyGrid();
@@ -239,28 +265,31 @@ void ImageViewerV2::displayImage(std::unique_ptr<QPixmap> _pixmap) {
     reset();
     if(_pixmap) {
         pixmapItemScaled.hide();
-
         pixmap = std::move(_pixmap);
         pixmap->setDevicePixelRatio(dpr);
         pixmapItem.setPixmap(*pixmap);
-
         Qt::TransformationMode mode = Qt::SmoothTransformation;
         if(mScalingFilter == QI_FILTER_NEAREST)
             mode = Qt::FastTransformation;
         pixmapItem.setTransformationMode(mode);
-
         pixmapItem.show();
-
         pixmapItem.setOffset((scene->width()  / 2.0) - (pixmap->width()  / (dpr * 2.0)),
                              (scene->height() / 2.0) - (pixmap->height() / (dpr * 2.0)));
         // always scale from center
         pixmapItem.setTransformOriginPoint(pixmapItem.boundingRect().center());
+        updateMinScale();
 
-        if(!keepFitMode)
-                imageFitMode = imageFitModeDefault;
-        if(imageFitMode == FIT_FREE)
-            imageFitMode = FIT_WINDOW;
-        applyFitMode();
+        if(!keepFitMode || imageFitMode == FIT_FREE)
+            imageFitMode = imageFitModeDefault;
+
+        if(mViewLock == LOCK_NONE) {
+            applyFitMode();
+        } else {
+            imageFitMode = FIT_FREE;
+            fitFree(lockedScale);
+            if(mViewLock == LOCK_ALL)
+                applySavedViewportPos();
+        }
         requestScaling();
 
         if(transparencyGridEnabled)
@@ -278,7 +307,7 @@ void ImageViewerV2::reset() {
     pixmapItem.setScale(1.0f);
     pixmap.reset();
     stopAnimation();
-    movie.reset(nullptr);
+    movie = nullptr;
     centerOn(sceneRect().center());
     // when this view is not in focus this it won't update the background
     // so we force it here
@@ -380,6 +409,7 @@ Qt::TransformationMode ImageViewerV2::selectTransformationMode() {
 
 void ImageViewerV2::setExpandImage(bool mode) {
     expandImage = mode;
+    updateMinScale();
     applyFitMode();
     requestScaling();
 }
@@ -431,12 +461,11 @@ void ImageViewerV2::drawTransparencyGrid() {
     */
 }
 
-// todo remove this?
 bool ImageViewerV2::imageFits() const {
     if(!pixmap)
         return true;
-    return (pixmap->width()  <= viewport()->width() &&
-            pixmap->height() <= viewport()->height());
+    return (pixmap->width()  <= (viewport()->width()  * devicePixelRatioF()) &&
+            pixmap->height() <= (viewport()->height() * devicePixelRatioF()));
 }
 
 bool ImageViewerV2::scaledImageFits() const {
@@ -546,6 +575,15 @@ void ImageViewerV2::mouseReleaseEvent(QMouseEvent *event) {
 // warning for future me:
 // for some reason in qgraphicsview wheelEvent is followed by moveEvent (wtf?)
 void ImageViewerV2::wheelEvent(QWheelEvent *event) {
+    #ifdef __APPLE__
+    // this event goes off during force touch with Qt::ScrollPhase being set to begin/end
+    // lets filter these
+    if(event->phase() == Qt::ScrollBegin || event->phase() == Qt::ScrollEnd) {
+        event->accept();
+        return;
+    }
+    #endif
+
     if(event->buttons() & Qt::RightButton) {
         event->accept();
         mouseInteraction = MOUSE_WHEEL_ZOOM;
@@ -555,37 +593,56 @@ void ImageViewerV2::wheelEvent(QWheelEvent *event) {
         else if(angleDelta < 0)
             zoomOutCursor();
     } else if(event->modifiers() == Qt::NoModifier) {
-        event->accept();
         QPoint pixelDelta = event->pixelDelta();
         QPoint angleDelta = event->angleDelta();
-        // high-precision touchpad
-        if(pixelDelta != QPoint(0,0) && settings->imageScrolling() != ImageScrolling::SCROLL_NONE) {
-            stopPosAnimation();
-            horizontalScrollBar()->setValue(horizontalScrollBar()->value() - pixelDelta.x());
-            verticalScrollBar()->setValue(verticalScrollBar()->value() - pixelDelta.y());
-            centerIfNecessary();
-            snapToEdges();
-        } else if(angleDelta != QPoint(0,0)) { // mouse wheel & (windows) touchpad
-            // wheel usually sends angleDelta = 120 / 240 / ...
-            // there doesnt seem to be a way to detect event source except this
-            // this issue is windows only as both linux touchpad drivers send pixelDelta instead
-            // as a workaround we use QElapsedTimer to guess where the event came from
-            bool isWheel = angleDelta.y() && !(angleDelta.y() % 120);
-            if(isWheel && lastTouchpadScroll.elapsed() > 100) {
-                if(settings->imageScrolling() == SCROLL_BY_TRACKPAD_AND_WHEEL)
-                    scroll(0, -angleDelta.y(), true);
-                else
-                    QWidget::wheelEvent(event);
-                return; // return immediately so we wont restart the scroll timer
-            } else if(settings->imageScrolling() != ImageScrolling::SCROLL_NONE) {
+        /* for reference
+         * linux
+         *   trackpad:
+         *     pixelDelta = (x,y) OR (0,0)
+         *     angleDelta = (x*scale,y*scale) OR (x,y)
+         *   wheel:
+         *     pixelDelta = (0,0)     - libinput <= 1.18
+         *     pixelDelta = (0,120*m) - libinput 1.19
+         *     angleDelta = (0,120*m)
+         * -----------------------------------------
+         * macOS
+         *   trackpad:
+         *     pixelDelta = (x,y)
+         *     angleDelta = (x*scale,y*scale)
+         *   wheel:
+         *     pixelDelta = (0,y*scrollAccel)
+         *     angleDelta = (0,120*m)
+         * -----------------------------------------
+         * windows
+         *   trackpad:
+         *     ?? (dont have the hardware with precision drivers)
+         *   wheel:
+         *     pixelDelta = (0,0)
+         *     AngleDelta = (0,120*m)
+         */
+        bool isWheel = angleDelta.y() && !(angleDelta.y() % 120) && lastTouchpadScroll.elapsed() > 100;
+        if(!isWheel) {
+            lastTouchpadScroll.restart();
+            event->accept();
+            if(settings->imageScrolling() != ImageScrolling::SCROLL_NONE) {
+                // scroll (high precision)
                 stopPosAnimation();
-                horizontalScrollBar()->setValue(horizontalScrollBar()->value() - angleDelta.x());
-                verticalScrollBar()->setValue(verticalScrollBar()->value() - angleDelta.y());
+                int dx = pixelDelta.x() ? pixelDelta.x() : angleDelta.x();
+                int dy = pixelDelta.y() ? pixelDelta.y() : angleDelta.y();
+                horizontalScrollBar()->setValue(horizontalScrollBar()->value() - dx * TRACKPAD_SCROLL_MULTIPLIER);
+                verticalScrollBar()->setValue(verticalScrollBar()->value()     - dy * TRACKPAD_SCROLL_MULTIPLIER);
                 centerIfNecessary();
                 snapToEdges();
             }
-            lastTouchpadScroll.restart();
+        } else if(isWheel && settings->imageScrolling() == SCROLL_BY_TRACKPAD_AND_WHEEL) {
+            // scroll by interval
+            event->accept();
+            scroll(0, -angleDelta.y(), true);
+        } else {
+           event->ignore();
+           QWidget::wheelEvent(event);
         }
+        saveViewportPos();
     } else {
         event->ignore();
         QWidget::wheelEvent(event);
@@ -609,6 +666,7 @@ void ImageViewerV2::mousePan(QMouseEvent *event) {
     mouseMoveStartPos -= event->pos();
     scroll(mouseMoveStartPos.x(), mouseMoveStartPos.y(), false);
     mouseMoveStartPos = event->pos();
+    saveViewportPos();
 }
 
 //  zooming while the right button is pressed
@@ -627,7 +685,34 @@ void ImageViewerV2::mouseMoveZoom(QMouseEvent *event) {
     zoomAnchored(newScale);
     centerIfNecessary();
     snapToEdges();
-    requestScaling();
+    if(pixmapItem.scale() == fitWindowScale)
+        imageFitMode = FIT_WINDOW;
+}
+
+// scale at which current image fills the window
+void ImageViewerV2::updateFitWindowScale() {
+    float newMinScaleX = (float) viewport()->width()  * devicePixelRatioF() / pixmap->width();
+    float newMinScaleY = (float) viewport()->height() * devicePixelRatioF() / pixmap->height();
+    if(newMinScaleX < newMinScaleY) {
+        fitWindowScale = newMinScaleX;
+    } else {
+        fitWindowScale = newMinScaleY;
+    }
+    if(expandImage && fitWindowScale > expandLimit)
+        fitWindowScale = expandLimit;
+}
+
+// limit min scale to window size
+void ImageViewerV2::updateMinScale() {
+    if(!pixmap)
+        return;
+    updateFitWindowScale();
+    if(imageFits())
+        minScale = 1.0f;
+    else
+        minScale = fitWindowScale;
+    if(mViewLock != LOCK_NONE && lockedScale < minScale)
+        minScale = lockedScale;
 }
 
 void ImageViewerV2::fitWidth() {
@@ -636,10 +721,12 @@ void ImageViewerV2::fitWidth() {
     float scaleX = (float)viewport()->width() * devicePixelRatioF() / pixmap->width();
     if(!expandImage && scaleX > 1.0f)
         scaleX = 1.0f;
-    swapToOriginalPixmap();
     if(scaleX > expandLimit)
         scaleX = expandLimit;
-    doZoom(scaleX);
+    if(currentScale() != scaleX) {
+        swapToOriginalPixmap();
+        doZoom(scaleX);
+    }
     centerIfNecessary();
     // just center somewhere at the top then do snap
     if(scaledSize().height() > viewport()->height()) {
@@ -653,33 +740,26 @@ void ImageViewerV2::fitWidth() {
 void ImageViewerV2::fitWindow() {
     if(!pixmap)
         return;
-    float scaleX = (float)viewport()->width()  * devicePixelRatioF() / pixmap->width();
-    float scaleY = (float)viewport()->height() * devicePixelRatioF() / pixmap->height();
-    if((scaleX < 1.0f) || (scaleY < 1.0f) || expandImage) {
-        // scaling to window
-        swapToOriginalPixmap();
-        float newScale;
-        if(scaleX < scaleY) {
-            // stretch to fill width
-            newScale = scaleX;
-        } else {
-            // stretch to fill height
-            newScale = scaleY;
-        }
-        if(newScale > expandLimit)
-            newScale = expandLimit;
-        doZoom(newScale);
-        centerOnPixmap();
-    } else {
+    if(imageFits() && !expandImage) {
         fitNormal();
+    } else {
+        if(currentScale() != fitWindowScale) {
+            swapToOriginalPixmap();
+            doZoom(fitWindowScale);
+        }
+        centerOnPixmap();
     }
 }
 
 void ImageViewerV2::fitNormal() {
+    fitFree(1.0f);
+}
+
+void ImageViewerV2::fitFree(float scale) {
     if(!pixmap)
         return;
     if(focusIn1to1 == FOCUS_TOP) {
-        doZoom(1.0f);
+        doZoom(scale);
         centerIfNecessary();
         if(scaledSize().height() > viewport()->height()) {
             QPointF centerTarget = sceneRect().center();
@@ -692,7 +772,7 @@ void ImageViewerV2::fitNormal() {
             setZoomAnchor(viewport()->rect().center());
         else
             setZoomAnchor(mapFromGlobal(cursor().pos()));
-        zoomAnchored(1.0f);
+        zoomAnchored(scale);
         centerIfNecessary();
         snapToEdges();
     }
@@ -749,6 +829,7 @@ void ImageViewerV2::resizeEvent(QResizeEvent *event) {
     // so we try to ignore them
     if(parentWidget()->isVisible()) {
         stopPosAnimation();
+        updateMinScale();
         if(imageFitMode == FIT_FREE || imageFitMode == FIT_ORIGINAL) {
             centerIfNecessary();
             snapToEdges();
@@ -759,6 +840,7 @@ void ImageViewerV2::resizeEvent(QResizeEvent *event) {
         if(scaleTimer->isActive())
             scaleTimer->stop();
         scaleTimer->start();
+        saveViewportPos();
     }
 }
 
@@ -833,6 +915,7 @@ void ImageViewerV2::scrollSmooth(int dx, int dy) {
         scrollTimeLineY->setFrameRange(currentYPos, newEndFrame);
         scrollTimeLineY->start();
     }
+    saveViewportPos();
 }
 
 void ImageViewerV2::scrollPrecise(int dx, int dy) {
@@ -841,6 +924,7 @@ void ImageViewerV2::scrollPrecise(int dx, int dy) {
     verticalScrollBar()->setValue(verticalScrollBar()->value() + dy);
     centerIfNecessary();
     snapToEdges();
+    saveViewportPos();
 }
 
 // used by scrollTimeLine
@@ -848,6 +932,8 @@ void ImageViewerV2::scrollToX(int x) {
     horizontalScrollBar()->setValue(x);
     centerIfNecessary();
     snapToEdges();
+    update();
+    qApp->processEvents();
 }
 
 // used by scrollTimeLine
@@ -855,6 +941,12 @@ void ImageViewerV2::scrollToY(int y) {
     verticalScrollBar()->setValue(y);
     centerIfNecessary();
     snapToEdges();
+    update();
+    qApp->processEvents();
+}
+
+void ImageViewerV2::onScrollTimelineFinished() {
+    saveViewportPos();
 }
 
 void ImageViewerV2::swapToOriginalPixmap() {
@@ -867,36 +959,108 @@ void ImageViewerV2::swapToOriginalPixmap() {
 }
 
 void ImageViewerV2::setZoomAnchor(QPoint viewportPos) {
-    zoomAnchor = QPair(pixmapItem.mapFromScene(mapToScene(viewportPos)),
+    zoomAnchor = QPair<QPointF, QPoint>(pixmapItem.mapFromScene(mapToScene(viewportPos)),
                        viewportPos);
 }
 
 void ImageViewerV2::zoomAnchored(float newScale) {
-    QPointF vportCenter = mapToScene(viewport()->geometry()).boundingRect().center();
-    doZoom(newScale);
-    // calculate shift to adjust viewport center
-    // we do this in viewport coordinates to avoid any rounding errors
-    QPointF diff = zoomAnchor.second - mapFromScene(pixmapItem.mapToScene(zoomAnchor.first));
-    centerOn(vportCenter - diff);
-    requestScaling();
- }
+    if(currentScale() != newScale) {
+        QPointF vportCenter = mapToScene(viewport()->geometry()).boundingRect().center();
+        doZoom(newScale);
+        // calculate shift to adjust viewport center
+        // we do this in viewport coordinates to avoid any rounding errors
+        QPointF diff = zoomAnchor.second - mapFromScene(pixmapItem.mapToScene(zoomAnchor.first));
+        centerOn(vportCenter - diff);
+        requestScaling();
+    }
+}
 
 // zoom in around viewport center
 void ImageViewerV2::zoomIn() {
     setZoomAnchor(viewport()->rect().center());
-    zoomAnchored(currentScale() * (1.0f + zoomStep));
+    if(absoluteStep)
+        zoomAnchored(currentScale() + zoomStep);
+    else
+        zoomAnchored(currentScale() * (1.0f + zoomStep));
     centerIfNecessary();
     snapToEdges();
     imageFitMode = FIT_FREE;
+    if(pixmapItem.scale() == fitWindowScale)
+        imageFitMode = FIT_WINDOW;
 }
 
 // zoom out around viewport center
 void ImageViewerV2::zoomOut() {
     setZoomAnchor(viewport()->rect().center());
-    zoomAnchored(currentScale() * (1.0f - zoomStep));
+    if(absoluteStep)
+        zoomAnchored(currentScale() - zoomStep);
+    else
+        zoomAnchored(currentScale() * (1.0f - zoomStep));
     centerIfNecessary();
     snapToEdges();
     imageFitMode = FIT_FREE;
+    if(pixmapItem.scale() == fitWindowScale)
+        imageFitMode = FIT_WINDOW;
+}
+
+void ImageViewerV2::toggleLockZoom() {
+    if(!isDisplaying())
+        return;
+    if(mViewLock != LOCK_ZOOM) {
+        mViewLock = LOCK_ZOOM;
+        lockZoom();
+    } else {
+        mViewLock = LOCK_NONE;
+    }
+}
+
+bool ImageViewerV2::lockZoomEnabled() {
+    return (mViewLock == LOCK_ZOOM);
+}
+
+void ImageViewerV2::lockZoom() {
+    lockedScale = pixmapItem.scale();
+    imageFitMode = FIT_FREE;
+    saveViewportPos();
+}
+
+void ImageViewerV2::toggleLockView() {
+    if(!isDisplaying())
+        return;
+    if(mViewLock != LOCK_ALL) {
+        mViewLock = LOCK_ALL;
+        lockZoom();
+        saveViewportPos();
+    } else {
+        mViewLock = LOCK_NONE;
+    }
+}
+
+bool ImageViewerV2::lockViewEnabled() {
+    return (mViewLock == LOCK_ALL);
+}
+
+// savedViewportPos is [0...1][0...1]
+// values are where viewport center is on the image
+void ImageViewerV2::saveViewportPos() {
+    if(mViewLock != LOCK_ALL)
+        return;
+    QGraphicsPixmapItem *item = &pixmapItem;
+    QPointF sceneCenter = mapToScene( viewport()->rect().center() ) + QPointF(1,1);
+    auto itemRect = item->sceneBoundingRect();
+    savedViewportPos.setX(qBound(qreal(0), (sceneCenter.x() - itemRect.left()) / itemRect.width(),  qreal(1)));
+    savedViewportPos.setY(qBound(qreal(0), (sceneCenter.y() - itemRect.top())  / itemRect.height(), qreal(1)));
+}
+
+void ImageViewerV2::applySavedViewportPos() {
+    QGraphicsPixmapItem *item = &pixmapItem;
+    auto itemRect = item->sceneBoundingRect();
+    QPointF newScenePos;
+    newScenePos.setX(itemRect.left() + itemRect.width()  * savedViewportPos.x());
+    newScenePos.setY(itemRect.top()  + itemRect.height() * savedViewportPos.y());
+    centerOn(newScenePos);
+    centerIfNecessary();
+    snapToEdges();
 }
 
 void ImageViewerV2::centerIfNecessary() {
@@ -932,14 +1096,20 @@ void ImageViewerV2::snapToEdges() {
     centerOn(centerTarget + QPointF(xShift, yShift));
 }
 
+// todo: just merge with zoomIn
 void ImageViewerV2::zoomInCursor() {
     if(underMouse()) {
         setZoomAnchor(mapFromGlobal(cursor().pos()));
-        zoomAnchored(currentScale() * (1.0f + zoomStep));
+        if(absoluteStep)
+            zoomAnchored(currentScale() + zoomStep);
+        else
+            zoomAnchored(currentScale() * (1.0f + zoomStep));
     } else {
         zoomIn();
     }
     imageFitMode = FIT_FREE;
+    if(pixmapItem.scale() == fitWindowScale)
+        imageFitMode = FIT_WINDOW;
     centerIfNecessary();
     snapToEdges();
 }
@@ -947,11 +1117,16 @@ void ImageViewerV2::zoomInCursor() {
 void ImageViewerV2::zoomOutCursor() {
     if(underMouse()) {
         setZoomAnchor(mapFromGlobal(cursor().pos()));
-        zoomAnchored(currentScale() * (1.0f - zoomStep));
+        if(absoluteStep)
+            zoomAnchored(currentScale() - zoomStep);
+        else
+            zoomAnchored(currentScale() * (1.0f - zoomStep));
     } else {
         zoomIn();
     }
     imageFitMode = FIT_FREE;
+    if(pixmapItem.scale() == fitWindowScale)
+        imageFitMode = FIT_WINDOW;
     centerIfNecessary();
     snapToEdges();
 }
@@ -959,7 +1134,7 @@ void ImageViewerV2::zoomOutCursor() {
 void ImageViewerV2::doZoom(float newScale) {
     if(!pixmap)
         return;
-    newScale = qBound(0.01f, newScale, 500.0f);
+    newScale = qBound(minScale, newScale, 500.0f);
     pixmapItem.setScale(newScale);
     pixmapItem.setTransformationMode(selectTransformationMode());
     swapToOriginalPixmap();
