@@ -14,6 +14,7 @@ ViewerWidget::ViewerWidget(QWidget *parent)
       videoControls(nullptr),
       currentWidget(UNSET),
       mInteractionEnabled(false),
+      mWaylandCursorWorkaround(false),
       avoidPanelFlag(false),
       mPanelEnabled(false),
       mPanelFullscreenOnly(false),
@@ -21,7 +22,12 @@ ViewerWidget::ViewerWidget(QWidget *parent)
 {
     setAttribute(Qt::WA_TranslucentBackground, true);
     setMouseTracking(true);
-
+#ifdef Q_OS_LINUX
+    // we cant check cursor position on wayland until the mouse is moved
+    // use this to skip cursor check once
+    if(qgetenv("XDG_SESSION_TYPE") == "wayland")
+        mWaylandCursorWorkaround = true;
+#endif
     layout.setContentsMargins(0, 0, 0, 0);
     layout.setSpacing(0);
     this->setLayout(&layout);
@@ -50,12 +56,9 @@ ViewerWidget::ViewerWidget(QWidget *parent)
 
     connect(videoPlayer.get(), &VideoPlayer::playbackFinished, this, &ViewerWidget::onVideoPlaybackFinished);
 
-    connect(videoControls, &VideoControlsProxyWrapper::pause,     this, &ViewerWidget::pauseResumePlayback);
-    connect(videoControls, &VideoControlsProxyWrapper::seekLeft,  this, &ViewerWidget::seekLeft);
-    connect(videoControls, &VideoControlsProxyWrapper::seekRight, this, &ViewerWidget::seekRight);
+    connect(videoControls, &VideoControlsProxyWrapper::seekBackward,  this, &ViewerWidget::seekBackward);
+    connect(videoControls, &VideoControlsProxyWrapper::seekForward, this, &ViewerWidget::seekForward);
     connect(videoControls, &VideoControlsProxyWrapper::seek,      this, &ViewerWidget::seek);
-    connect(videoControls, &VideoControlsProxyWrapper::nextFrame, this, &ViewerWidget::frameStep);
-    connect(videoControls, &VideoControlsProxyWrapper::prevFrame, this, &ViewerWidget::frameStepBack);
 
     enableImageViewer();
     enableInteraction();
@@ -137,7 +140,8 @@ void ViewerWidget::disableVideoPlayer() {
         // which paints over the imageviewer, causing corruption
         // so we do not HIDE it, but rather just cover it by imageviewer's widget
         // seems to work fine, might even feel a bit snappier
-        //videoPlayer->hide();
+        if(!videoPlayer->isInitialized())
+            videoPlayer->hide();
     }
 }
 
@@ -148,7 +152,7 @@ void ViewerWidget::onScaleChanged(qreal scale) {
         zoomIndicator->setScale(scale);
         if(settings->zoomIndicatorMode() == ZoomIndicatorMode::INDICATOR_ENABLED)
             zoomIndicator->show();
-        else if((settings->zoomIndicatorMode() == ZoomIndicatorMode::INDICATOR_AUTOHIDE))
+        else if((settings->zoomIndicatorMode() == ZoomIndicatorMode::INDICATOR_AUTO))
             zoomIndicator->show(1500);
     } else {
         zoomIndicator->hide();
@@ -167,6 +171,8 @@ void ViewerWidget::onAnimationPlaybackFinished() {
 
 void ViewerWidget::enableInteraction() {
     if(!mInteractionEnabled) {
+        connect(this, &ViewerWidget::toggleLockZoom, imageViewer.get(), &ImageViewerV2::toggleLockZoom);
+        connect(this, &ViewerWidget::toggleLockView, imageViewer.get(), &ImageViewerV2::toggleLockView);
         connect(this, &ViewerWidget::zoomIn,        imageViewer.get(), &ImageViewerV2::zoomIn);
         connect(this, &ViewerWidget::zoomOut,       imageViewer.get(), &ImageViewerV2::zoomOut);
         connect(this, &ViewerWidget::zoomInCursor,  imageViewer.get(), &ImageViewerV2::zoomInCursor);
@@ -208,7 +214,7 @@ bool ViewerWidget::interactionEnabled() {
     return mInteractionEnabled;
 }
 
-std::shared_ptr<ThumbnailStrip> ViewerWidget::getThumbPanel() {
+std::shared_ptr<ThumbnailStripProxy> ViewerWidget::getThumbPanel() {
     return mainPanel->getThumbnailStrip();
 }
 
@@ -223,12 +229,12 @@ bool ViewerWidget::showImage(std::unique_ptr<QPixmap> pixmap) {
     return true;
 }
 
-bool ViewerWidget::showAnimation(std::unique_ptr<QMovie> movie) {
+bool ViewerWidget::showAnimation(std::shared_ptr<QMovie> movie) {
     if(!movie)
         return false;
     stopPlayback();
     enableImageViewer();
-    imageViewer->displayAnimation(std::move(movie));
+    imageViewer->displayAnimation(movie);
     hideCursorTimed(false);
     return true;
 }
@@ -307,12 +313,12 @@ void ViewerWidget::seekRelative(int pos) {
         videoPlayer.get()->seekRelative(pos);
 }
 
-void ViewerWidget::seekLeft() {
+void ViewerWidget::seekBackward() {
     if(currentWidget == VIDEOPLAYER)
         videoPlayer.get()->seekRelative(-10);
 }
 
-void ViewerWidget::seekRight() {
+void ViewerWidget::seekForward() {
     if(currentWidget == VIDEOPLAYER)
         videoPlayer.get()->seekRelative(10);
 }
@@ -363,6 +369,14 @@ bool ViewerWidget::isDisplaying() {
         return false;
 }
 
+bool ViewerWidget::lockZoomEnabled() {
+    return imageViewer->lockZoomEnabled();
+}
+
+bool ViewerWidget::lockViewEnabled() {
+    return imageViewer->lockViewEnabled();
+}
+
 ScalingFilter ViewerWidget::scalingFilter() {
     return imageViewer->scalingFilter();
 }
@@ -383,6 +397,11 @@ bool ViewerWidget::panelEnabled() {
     return mPanelEnabled;
 }
 
+void ViewerWidget::setupMainPanel() {
+    if(mPanelEnabled)
+        mainPanel->setupThumbnailStrip();
+}
+
 void ViewerWidget::mousePressEvent(QMouseEvent *event) {
     hideContextMenu();
     event->ignore();
@@ -395,6 +414,7 @@ void ViewerWidget::mouseReleaseEvent(QMouseEvent *event) {
 }
 
 void ViewerWidget::mouseMoveEvent(QMouseEvent *event) {
+    mWaylandCursorWorkaround = false;
     if(!(event->buttons() & Qt::LeftButton) && !(event->buttons() & Qt::RightButton)) {
         showCursor();
         hideCursorTimed(true);
@@ -451,13 +471,19 @@ void ViewerWidget::hideCursor() {
     // ignore when menu is up
     if(contextMenu && contextMenu->isVisible())
         return;
-    // only hide when we are under viewer or player widget
     if(settings->cursorAutohide()) {
-        QWidget *w = qApp->widgetAt(QCursor::pos());
-        if(w && (w == imageViewer.get()->viewport() || w == videoPlayer->getPlayer().get())) {
-            if(!videoControls->isVisible() || !videoControlsArea().contains(mapFromGlobal(QCursor::pos()))) {
-                setCursor(QCursor(Qt::BlankCursor));
-                videoControls->hide();
+        // force hide on wayland until we can get the cursor pos
+        if(mWaylandCursorWorkaround) {
+            setCursor(QCursor(Qt::BlankCursor));
+            videoControls->hide();
+        } else {
+            // only hide when we are under viewer or player widget
+            QWidget *w = qApp->widgetAt(QCursor::pos());
+            if(w && (w == imageViewer.get()->viewport() || w == videoPlayer->getPlayer().get())) {
+                if(!videoControls->isVisible() || !videoControlsArea().contains(mapFromGlobal(QCursor::pos()))) {
+                    setCursor(QCursor(Qt::BlankCursor));
+                    videoControls->hide();
+                }
             }
         }
     }
@@ -465,7 +491,7 @@ void ViewerWidget::hideCursor() {
 
 QRect ViewerWidget::videoControlsArea() {
     QRect vcontrolsRect;
-    if(mainPanel->position() == PANEL_TOP)
+    if(!mPanelEnabled || mainPanel->position() == PANEL_TOP)
         vcontrolsRect = QRect(0, height() - 160, width(), height());
     else
         vcontrolsRect = QRect(0, 0, width(), 160);
@@ -526,16 +552,20 @@ void ViewerWidget::hideEvent(QHideEvent *event) {
     hideContextMenu();
 }
 
-void ViewerWidget::keyPressEvent(QKeyEvent *event) {
-    if(currentWidget == VIDEOPLAYER && event->key() == Qt::Key_Space) {
-        event->accept();
-        videoPlayer->pauseResume();
-    } else {
-        event->ignore();
-    }
+// block native tab-switching so we can use it in shortcuts
+bool ViewerWidget::focusNextPrevChild(bool mode) {
+    return false;
 }
 
-void ViewerWidget::enterEvent(QEvent *event) {
+void ViewerWidget::keyPressEvent(QKeyEvent *event) {
+    event->accept();
+    if(currentWidget == VIDEOPLAYER && event->key() == Qt::Key_Space)
+        videoPlayer->pauseResume();
+    else
+        actionManager->processEvent(event);
+}
+
+void ViewerWidget::enterEvent(QEnterEvent *event) {
     QWidget::enterEvent(event);
 
     if(!mInteractionEnabled)
