@@ -3,19 +3,17 @@
 #include <QImage>
 #include <QPixmap>
 #include <QProcess>
+#include <QRegExp>
 #include <QFile>
 #include "image.h"
-#include "clip.h"
 
 class Video : public Image {
 public:
     Video(QString _path);
     Video(std::unique_ptr<DocumentInfo> _info);
-    ~Video();
 
     std::unique_ptr<QPixmap> getPixmap();
     std::shared_ptr<const QImage> getImage();
-    Clip* getClip();    // getPixmap's video equivalent
     int height();
     int width();
     QSize size();
@@ -27,5 +25,7 @@ public slots:
 
 private:
     void load();
-    Clip *clip;
+
+    uint srcWidth = 0;
+    uint srcHeight = 0;
 };
